/**
 * `IxState` (Indexed State) is a derivation of `State`, that may have **polymorphic** states;
 * The input may be of a different type than the output.
 *
 * State changes are kept track of by the generics `I`, `O`, `X` and `Z`.
 * - `I` indicates the current input state
 * - `O` indicates the current output state
 * - `Z` indicates the next input state
 * - `Z` indicates the next output state
 *
 * When composing `IxState` where the indexes are polymorhpic, functions `ixchain` and `ixap` can be
 * used to compose `<I, O>` with `<O, Z>` to return `<I, Z>`.
 *
 * Intuitively this is function composition on the indexes.
 *
 *
 * `IxState` derived by applying the `Identity` monad to the transformer `IxStateT`.
 * @since 2.10.0
 */
import { Apply2 } from 'fp-ts/Apply'
import { Chain2 } from 'fp-ts/Chain'
import { Functor3 } from 'fp-ts/Functor'
import { Monad2 } from 'fp-ts/Monad'
import { Pointed2 } from 'fp-ts/Pointed'
import { State } from 'fp-ts/State'
import { IxApplicative3 } from './IxApplicative'
import { IxApply3 } from './IxApply'
import { IxChain3 } from './IxChain'
import { IxFunctor3 } from './IxFunctor'
import { IxMonad3 } from './IxMonad'
import { IxPointed3 } from './IxPointed'
import * as IxStateT from './IxStateT'
/**
 * @category Model
 * @since 2.10.0
 */
export interface IxState<I, O, A> {
  (i: I): [A, O]
}
/**
 * @category Model
 * @since 2.10.0
 */
export declare const URI = 'IxState'
/**
 * @category Model
 * @since 2.10.0
 */
export declare type URI = typeof URI
declare module 'fp-ts/HKT' {
  interface URItoKind3<R, E, A> {
    readonly [URI]: IxState<R, E, A>
  }
  interface URItoKind2<E, A> {
    readonly [URI]: IxState<E, E, A>
  }
}
/**
 * @summary
 * Apply a function to the state, polymorphically changing the resulting state.
 *
 * @category constructors
 * @since 2.10.0
 */
export declare const ixmodify: <I, O>(f: (i: I) => O) => IxState<I, O, void>
/**
 * @category constructors
 * @since 2.10.0
 */
export declare const put: <I>(fi: I) => IxState<I, I, void>
/**
 * @category constructors
 * @since 2.10.0
 */
export declare const fromState: <I, A>(fa: State<I, A>) => IxState<I, I, A>
/**
 * @category constructors
 * @since 2.10.0
 */
export declare const get: <I>() => IxStateT.IxStateT1<'Identity', I, I, I>
/**
 * @category constructors
 * @since 2.10.0
 */
export declare const gets: <I, A>(f: (i: I) => A) => IxStateT.IxStateT1<'Identity', I, I, A>
/**
 * @category destructors
 * @since 2.10.0
 */
export declare const toState: <I, A>(fa: IxState<I, I, A>) => State<I, A>
/**
 * @category destructors
 * @since 2.10.0
 */
export declare const ixexecute: <I>(i: I) => <O, A>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => O
/**
 * @category destructors
 * @since 2.10.0
 */
export declare const evaluate: <I>(i: I) => <O, A>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => A
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxFunctor: IxFunctor3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxApply: IxApply3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxChain: IxChain3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxPointed: IxPointed3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxApplicative: IxApplicative3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const IxMonad: IxMonad3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const Pointed: Pointed2<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const Functor: Functor3<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const Apply: Apply2<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const Chain: Chain2<URI>
/**
 * @category instances
 * @since 2.10.0
 */
export declare const Monad: Monad2<URI>
/**
 * @category IxPointed
 * @since 2.10.0
 */
export declare const ixof: <I, A>(a: A) => IxState<I, I, A>
/**
 * @category IxFunctor
 * @since 2.10.0
 */
export declare const ixmap: <A, B>(
  f: (a: A) => B
) => <I, O>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => IxStateT.IxStateT1<'Identity', I, O, B>
/**
 * @category IxFunctor
 * @since 2.10.0
 */
export declare const ixflap: <A>(a: A) => <I, O, B>(fab: IxState<I, O, (a: A) => B>) => IxState<I, O, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category IxApply
 * @since 2.10.0
 */
export declare const ixap: <O, Z, A>(
  fa: IxStateT.IxStateT1<'Identity', O, Z, A>
) => <I, B>(fab: IxStateT.IxStateT1<'Identity', I, O, (a: A) => B>) => IxStateT.IxStateT1<'Identity', I, Z, B>
/**
 * @category IxApply
 *
 * @since 2.10.0
 */
export declare const ixapFirst: <O, Z, B>(
  second: IxState<O, Z, B>
) => <I, A>(first: IxState<I, O, A>) => IxState<I, Z, A>
/**
 * @category IxApply
 * @since 2.10.0
 */
export declare const ixapSecond: <I, O, Z, A, B>(
  second: IxState<O, Z, B>
) => (first: IxState<I, O, A>) => IxState<I, Z, B>
/**
 * @category Apply
 *
 * @since 2.10.0
 */
export declare const apFirst: <E, B>(second: IxState<E, E, B>) => <A>(first: IxState<E, E, A>) => IxState<E, E, A>
/**
 * @category Apply
 * @since 2.10.0
 */
export declare const apSecond: <E, B>(second: IxState<E, E, B>) => <A>(first: IxState<E, E, A>) => IxState<E, E, B>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category IxChain
 *
 * @since 2.10.0
 */
export declare const ixchain: <O, Z, A, B>(
  f: (a: A) => IxStateT.IxStateT1<'Identity', O, Z, B>
) => <I>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => IxStateT.IxStateT1<'Identity', I, Z, B>
/**
 * @category IxChain
 * @since 2.10.0
 */
export declare const ixchainFirst: <I, O, Z, A, B>(
  f: (a: A) => IxState<O, Z, B>
) => (first: IxState<I, O, A>) => IxState<I, Z, A>
/**
 * @category Chain
 * @since 2.10.0
 */
export declare const chainFirst: <A, E, B>(
  f: (a: A) => IxState<E, E, B>
) => (first: IxState<E, E, A>) => IxState<E, E, A>
/**
 * @category Pointed
 * @since 2.10.0
 */
export declare const of: Pointed2<URI>['of']
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 2.10.0
 */
export declare const map: <A, B>(
  f: (a: A) => B
) => <I, O>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => IxStateT.IxStateT1<'Identity', I, O, B>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 2.10.0
 */
export declare const flap: <A>(a: A) => <R, E, B>(fab: IxState<R, E, (a: A) => B>) => IxState<R, E, B>
/**
 *
 * @category Apply
 * @since 2.10.0
 */
export declare const ap: <I, A>(
  fa: IxStateT.IxStateT1<'Identity', I, I, A>
) => <B>(fab: IxStateT.IxStateT1<'Identity', I, I, (a: A) => B>) => IxStateT.IxStateT1<'Identity', I, I, B>
/**
 *
 * @category Chain
 * @since 2.10.0
 */
export declare const chain: <I, A, B>(
  f: (a: A) => IxStateT.IxStateT1<'Identity', I, I, B>
) => (fa: IxStateT.IxStateT1<'Identity', I, I, A>) => IxStateT.IxStateT1<'Identity', I, I, B>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 2.10.0
 */
export declare const local: <I, X>(
  f: (x: X) => I
) => <O, A>(fa: IxStateT.IxStateT1<'Identity', I, O, A>) => IxStateT.IxStateT1<'Identity', X, O, A>
/**
 * @since 2.10.0
 */
export declare const ixDo: <I>() => IxState<I, I, {}>
/**
 * @since 2.10.0
 */
export declare const Do: <I>() => IxState<I, I, {}>
/**
 * @since 2.10.0
 */
export declare const ixbind: <N extends string, A, Second, Third, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => IxState<Second, Third, B>
) => <First>(
  ma: IxState<First, Second, A>
) => IxState<First, Third, { [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 2.10.0
 */
export declare const ixbindTo: <N extends string>(
  name: N
) => <I, O, A>(fa: IxState<I, O, A>) => IxState<I, O, { [K in N]: A }>
/**
 * @since 2.10.0
 */
export declare const bind: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => IxState<E, E, B>
) => (ma: IxState<E, E, A>) => IxState<E, E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 2.10.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <E, A>(fa: IxState<E, E, A>) => IxState<E, E, { readonly [K in N]: A }>
/**
 * ixDo-notation combinator for discarding the result of a computation rather
 * than binding it to a name. It is simply an alias for `ixchainFirst`
 * @since 2.10.0
 */
export declare const ixdiscard: <I, O, Z, A, B>(
  f: (a: A) => IxState<O, Z, B>
) => (first: IxState<I, O, A>) => IxState<I, Z, A>
/**
 * Do-notation combinator for discarding the result of a computation rather
 * than binding it to a name. It is simply an alias for `chainFirst`
 * @since 2.10.0
 */
export declare const discard: <A, E, B>(f: (a: A) => IxState<E, E, B>) => (first: IxState<E, E, A>) => IxState<E, E, A>
/**
 * @since 2.10.0
 */
export declare const ixapS: <N extends string, A, O, S, B>(
  name: Exclude<N, keyof A>,
  fb: IxState<O, S, B>
) => <First, O_1>(fa: IxState<First, O_1, A>) => IxState<First, S, { [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 2.10.0
 */
export declare const apS: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  fb: IxState<E, E, B>
) => (fa: IxState<E, E, A>) => IxState<E, E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
